@@ -100,6 +100,7 @@ struct GMenuTreeDirectory
 
   guint only_unallocated : 1;
   guint is_root : 1;
+  guint is_nodisplay : 1;
 };
 
 typedef struct
@@ -117,6 +118,7 @@ struct GMenuTreeEntry
   char         *desktop_file_id;
 
   guint is_excluded : 1;
+  guint is_nodisplay : 1;
 };
 
 struct GMenuTreeSeparator
@@ -744,13 +746,13 @@ find_path (GMenuTreeDirectory *directory,
   char       *freeme;
   GSList     *tmp;
 
-  while (path[0] == '/') path++;
+  while (path[0] == G_DIR_SEPARATOR) path++;
 
   if (path[0] == '\0')
     return directory;
 
   freeme = NULL;
-  slash = strchr (path, '/');
+  slash = strchr (path, G_DIR_SEPARATOR);
   if (slash)
     {
       name = freeme = g_strndup (path, slash - path);
@@ -768,7 +770,10 @@ find_path (GMenuTreeDirectory *directory,
       GMenuTreeItem *item = tmp->data;
 
       if (gmenu_tree_item_get_type (item) != GMENU_TREE_ITEM_DIRECTORY)
-	continue;
+        {
+          tmp = tmp->next;
+          continue;
+        }
 
       if (!strcmp (name, GMENU_TREE_DIRECTORY (item)->name))
 	{
@@ -798,7 +803,7 @@ gmenu_tree_get_directory_from_path (GMenuTree  *tree,
   g_return_val_if_fail (tree != NULL, NULL);
   g_return_val_if_fail (path != NULL, NULL);
 
-  if (path[0] != '/')
+  if (path[0] != G_DIR_SEPARATOR)
     return NULL;
 
   if (!(root = gmenu_tree_get_root_directory (tree)))
@@ -969,6 +974,17 @@ gmenu_tree_directory_get_icon (GMenuTreeDirectory *directory)
   return desktop_entry_get_icon (directory->directory_entry);
 }
 
+const char *
+gmenu_tree_directory_get_desktop_file_path (GMenuTreeDirectory *directory)
+{
+  g_return_val_if_fail (directory != NULL, NULL);
+
+  if (!directory->directory_entry)
+    return NULL;
+
+  return desktop_entry_get_path (directory->directory_entry);
+}
+
 const char *
 gmenu_tree_directory_get_menu_id (GMenuTreeDirectory *directory)
 {
@@ -1009,6 +1025,14 @@ gmenu_tree_directory_get_tree (GMenuTreeDirectory *directory)
   return gmenu_tree_ref (root->tree);
 }
 
+gboolean
+gmenu_tree_directory_get_is_nodisplay (GMenuTreeDirectory *directory)
+{
+  g_return_val_if_fail (directory != NULL, FALSE);
+
+  return directory->is_nodisplay;
+}
+
 static void
 append_directory_path (GMenuTreeDirectory *directory,
 		       GString            *path)
@@ -1016,14 +1040,14 @@ append_directory_path (GMenuTreeDirectory *directory,
 
   if (!directory->item.parent)
     {
-      g_string_append_c (path, '/');
+      g_string_append_c (path, G_DIR_SEPARATOR);
       return;
     }
 
   append_directory_path (directory->item.parent, path);
 
   g_string_append (path, directory->name);
-  g_string_append_c (path, '/');
+  g_string_append_c (path, G_DIR_SEPARATOR);
 }
 
 char *
@@ -1077,6 +1101,14 @@ gmenu_tree_entry_get_exec (GMenuTreeEntry *entry)
   return desktop_entry_get_exec (entry->desktop_entry);
 }
 
+gboolean
+gmenu_tree_entry_get_launch_in_terminal (GMenuTreeEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, FALSE);
+
+  return desktop_entry_get_launch_in_terminal (entry->desktop_entry);
+}
+
 const char *
 gmenu_tree_entry_get_desktop_file_path (GMenuTreeEntry *entry)
 {
@@ -1101,6 +1133,14 @@ gmenu_tree_entry_get_is_excluded (GMenuTreeEntry *entry)
   return entry->is_excluded;
 }
 
+gboolean
+gmenu_tree_entry_get_is_nodisplay (GMenuTreeEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, FALSE);
+
+  return entry->is_nodisplay;
+}
+
 GMenuTreeDirectory *
 gmenu_tree_header_get_directory (GMenuTreeHeader *header)
 {
@@ -1160,6 +1200,7 @@ gmenu_tree_directory_new (GMenuTreeDirectory *parent,
   retval->layout_info         = NULL;
   retval->contents            = NULL;
   retval->only_unallocated    = FALSE;
+  retval->is_nodisplay        = FALSE;
 
   if (parent != NULL)
     {
@@ -1324,7 +1365,8 @@ static GMenuTreeEntry *
 gmenu_tree_entry_new (GMenuTreeDirectory *parent,
 		      DesktopEntry       *desktop_entry,
 		      const char         *desktop_file_id,
-		      gboolean            is_excluded)
+		      gboolean            is_excluded,
+                      gboolean            is_nodisplay)
 {
   GMenuTreeEntry *retval;
 
@@ -1337,6 +1379,7 @@ gmenu_tree_entry_new (GMenuTreeDirectory *parent,
   retval->desktop_entry   = desktop_entry_ref (desktop_entry);
   retval->desktop_file_id = g_strdup (desktop_file_id);
   retval->is_excluded     = is_excluded != FALSE;
+  retval->is_nodisplay    = is_nodisplay != FALSE;
 
   return retval;
 }
@@ -2143,12 +2186,12 @@ add_menu_for_legacy_dir (MenuLayoutNode *parent,
         {
           const char *subdir = tmp->data;
 
-          g_string_append_c (subdir_path, '/');
+          g_string_append_c (subdir_path, G_DIR_SEPARATOR);
           g_string_append (subdir_path, subdir);
 
 	  if (relative_path_len)
 	    {
-	      g_string_append_c (subdir_relative, '/');
+	      g_string_append_c (subdir_relative, G_DIR_SEPARATOR);
 	    }
           g_string_append (subdir_relative, subdir);
 
@@ -2551,10 +2594,10 @@ find_submenu (MenuLayoutNode *layout,
 
   menu_verbose (" (splitting \"%s\")\n", path);
 
-  if (path[0] == '\0' || path[0] == '/')
+  if (path[0] == '\0' || path[0] == G_DIR_SEPARATOR)
     return NULL;
 
-  slash = strchr (path, '/');
+  slash = strchr (path, G_DIR_SEPARATOR);
   if (slash != NULL)
     {
       name = g_strndup (path, slash - path);
@@ -2643,7 +2686,6 @@ gmenu_tree_execute_moves (GMenuTree      *tree,
   MenuLayoutNode *child;
   gboolean        need_remove_dups;
   GSList         *move_nodes;
-  GSList         *prev;
   GSList         *tmp;
 
   need_remove_dups = FALSE;
@@ -2673,7 +2715,6 @@ gmenu_tree_execute_moves (GMenuTree      *tree,
       child = menu_layout_node_get_next (child);
     }
 
-  prev = NULL;
   tmp = move_nodes;
   while (tmp != NULL)
     {
@@ -3008,9 +3049,10 @@ entries_listify_foreach (const char         *desktop_file_id,
   directory->entries =
     g_slist_prepend (directory->entries,
 		     gmenu_tree_entry_new (directory,
-					  desktop_entry,
-					  desktop_file_id,
-					  FALSE));
+                                           desktop_entry,
+                                           desktop_file_id,
+                                           FALSE,
+                                           desktop_entry_get_no_display (desktop_entry)));
 }
 
 static void
@@ -3023,7 +3065,8 @@ excluded_entries_listify_foreach (const char         *desktop_file_id,
 		     gmenu_tree_entry_new (directory,
 					   desktop_entry,
 					   desktop_file_id,
-					   TRUE));
+					   TRUE,
+                                           desktop_entry_get_no_display (desktop_entry)));
 }
 
 static GMenuTreeDirectory *
@@ -3241,9 +3284,14 @@ process_layout (GMenuTree          *tree,
     {
       if (desktop_entry_get_no_display (directory->directory_entry))
         {
-          menu_verbose ("Not showing menu %s because NoDisplay=true\n",
+          directory->is_nodisplay = TRUE;
+
+          if (!(tree->flags & GMENU_TREE_FLAGS_INCLUDE_NODISPLAY))
+            {
+              menu_verbose ("Not showing menu %s because NoDisplay=true\n",
                         desktop_entry_get_name (directory->directory_entry));
-          deleted = TRUE;
+              deleted = TRUE;
+            }
         }
 
       if (!desktop_entry_get_show_in_gnome (directory->directory_entry))
@@ -3290,7 +3338,8 @@ process_layout (GMenuTree          *tree,
           delete = TRUE;
         }
 
-      if (desktop_entry_get_no_display (entry->desktop_entry))
+      if (!(tree->flags & GMENU_TREE_FLAGS_INCLUDE_NODISPLAY) &&
+          desktop_entry_get_no_display (entry->desktop_entry))
         {
           menu_verbose ("Deleting %s because NoDisplay=true\n",
                         desktop_entry_get_name (entry->desktop_entry));
